//
//  RootViewController.m
//  Demine
//
//  Created by Michael Heyeck on 4/9/10.
//  Copyright Fair Oaks Labs, Inc. 2010. All rights reserved.
//

#import "Puzzle.h"
#import "ManagedPuzzle.h"
#import "NewPuzzle.h"
#import "HighScores.h"
#import "Playfield.h"
#import "RootViewController.h"


@interface RootViewController ()

@property (nonatomic, retain, readonly) NSDateFormatter* timerFormatter;

- (void)insertNewObject;
- (NSString*)formatTimer:(NSTimeInterval)timeInterval;
- (void)showHighScores;

@end


@implementation RootViewController

@synthesize fetchedResultsController, managedObjectContext;

- (NSDateFormatter*)timerFormatter
{
	if (!timerFormatter)
	{
		timerFormatter = [[NSDateFormatter alloc] init];
		timerFormatter.dateFormat = @"H:mm:ss";
		timerFormatter.timeZone = [NSTimeZone timeZoneForSecondsFromGMT:0];
	}
	return timerFormatter;
}


#pragma mark -
#pragma mark View lifecycle


- (void)viewDidLoad
{
    [super viewDidLoad];

	// Set up the title
	self.title = @"Saved Minefields";
	self.navigationItem.backBarButtonItem = [[[UIBarButtonItem alloc] initWithTitle:@"Pause" style:UIBarButtonItemStyleBordered target:nil action:nil] autorelease];

	// Set up the high score and add buttons.
	self.navigationItem.leftBarButtonItem = [[[UIBarButtonItem alloc] initWithTitle:@"Best" style:UIBarButtonItemStyleBordered target:self action:@selector(showHighScores)] autorelease];
    
    UIBarButtonItem *addButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAdd target:self action:@selector(insertNewObject)];
    self.navigationItem.rightBarButtonItem = addButton;
    [addButton release];
	
	NSError *error = nil;
	if (![[self fetchedResultsController] performFetch:&error])
	{
		NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
		
		UIAlertView* av = [[UIAlertView alloc] initWithTitle:nil
													 message:@"An unrecoverable error occurred.  Please press the Home button to exit."
													delegate:nil
										   cancelButtonTitle:nil
										   otherButtonTitles:nil];
		[av show];
		[av release];
	}
	
	if (![self tableView:self.tableView numberOfRowsInSection:0])
	{
		[self insertNewObject];
	}
}

/*
- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
}
*/
/*
- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
}
*/
/*
- (void)viewWillDisappear:(BOOL)animated {
	[super viewWillDisappear:animated];
}
*/
/*
- (void)viewDidDisappear:(BOOL)animated {
	[super viewDidDisappear:animated];
}
*/

- (void)viewDidUnload {
	// Relinquish ownership of anything that can be recreated in viewDidLoad or on demand.
	// For example: self.myOutlet = nil;
}

/*
 // Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
	// Return YES for supported orientations.
	return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
 */


- (id)addPuzzleWithWidth:(NSUInteger)w height:(NSUInteger)h mines:(NSUInteger)m
{
	// Build and generate puzzle
	Puzzle* p = [[[Puzzle alloc] initWithWidth:w height:h mines:m] autorelease];
	[p generate];
	
	// Create a new managed puzzle
	ManagedPuzzle* mp = [NSEntityDescription insertNewObjectForEntityForName:@"ManagedPuzzle" inManagedObjectContext:managedObjectContext];
	
	// Configure the new managed object.
	[p saveToManagedPuzzle:mp];
	
	// Save the context
    NSError* error = nil;
    if (![managedObjectContext save:&error])
	{
		[managedObjectContext deleteObject:mp];
		UIAlertView* av = [[UIAlertView alloc] initWithTitle:nil
													 message:@"Puzzle creation unexpectedly failed!"
													delegate:nil
										   cancelButtonTitle:@"Ok"
										   otherButtonTitles:nil];
		[av show];
		[av release];
		return nil;
    }
	
	// Present the puzzle
	Playfield* playfield = [[Playfield alloc] initWithNibName:@"Playfield" bundle:nil puzzle:mp];
	[self.navigationController pushViewController:playfield animated:NO];
	[playfield release];
	
	// Return the (in-memory) puzzle
	return p;
}

#pragma mark -
#pragma mark Add a new object

- (void)insertNewObject
{
	NewPuzzle* np = [[[NewPuzzle alloc] initWithNibName:@"NewPuzzle" bundle:nil target:self] autorelease];
	UINavigationController* nc = [[[UINavigationController alloc] initWithRootViewController:np] autorelease];
	[self.navigationController presentModalViewController:nc animated:YES];
}

#pragma mark -
#pragma mark Table view methods

- (NSInteger)numberOfSectionsInTableView:(UITableView*)tableView
{
    return 1;
}


// Customize the number of rows in the table view.
- (NSInteger)tableView:(UITableView*)tableView numberOfRowsInSection:(NSInteger)section
{
	NSArray* sections = [fetchedResultsController sections];
	if ([sections count])
	{
		return [[sections objectAtIndex:section] numberOfObjects];
	}
	else
	{
		return 0;
	}
}


// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    static NSString *CellIdentifier = @"Cell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier] autorelease];
		cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
    }
    
	ManagedPuzzle* mp = [fetchedResultsController objectAtIndexPath:indexPath];
	NSString* size = [NSString stringWithFormat:@"%d x %d (%d)",[mp.w integerValue],[mp.h integerValue],[mp.nMines integerValue]];
	NSString* time = [self formatTimer:[mp.elapsedTime doubleValue]];
	float percent = 100.0 * [mp.nUncovered floatValue] / ([mp.w floatValue]*[mp.h floatValue] - [mp.nMines floatValue]);
	
	cell.textLabel.text = size;
	if ([mp.status integerValue] == PuzzleStatusSolved)
	{
		cell.detailTextLabel.text = [NSString stringWithFormat:@"%@ (Solved!)",time];
		cell.detailTextLabel.textColor = [UIColor greenColor];
	}
	else if ([mp.status integerValue] == PuzzleStatusFailed)
	{
		cell.detailTextLabel.text = [NSString stringWithFormat:@"%@ (Failed)",time];
		cell.detailTextLabel.textColor = [UIColor redColor];
	}
	else
	{
		cell.detailTextLabel.text = [NSString stringWithFormat:@"%@ (%.0f%%)",time,percent];
		cell.detailTextLabel.textColor = [UIColor grayColor];
	}
	
    return cell;
}


- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
	// Get the selected object (to pass to the new view controller).
	ManagedPuzzle* mp = [[self fetchedResultsController] objectAtIndexPath:indexPath];
	
    // Navigation logic may go here -- for example, create and push another view controller.
	Playfield* playfield = [[Playfield alloc] initWithNibName:@"Playfield" bundle:nil puzzle:mp];
	[self.navigationController pushViewController:playfield animated:YES];
	[playfield release];
}


/*
// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the specified item to be editable.
    return YES;
}
*/


// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        // Delete the managed object for the given index path
		NSManagedObjectContext *context = [fetchedResultsController managedObjectContext];
		[context deleteObject:[fetchedResultsController objectAtIndexPath:indexPath]];
		
		// Save the context.
		NSError *error = nil;
		if (![context save:&error]) {
			/*
			 Replace this implementation with code to handle the error appropriately.
			 */
			NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
		}
	}   
}


- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath {
    // The table view should not be re-orderable.
    return NO;
}


#pragma mark -
#pragma mark Fetched results controller

- (NSFetchedResultsController *)fetchedResultsController {
    
    if (fetchedResultsController != nil) {
        return fetchedResultsController;
    }
    
    /*
	 Set up the fetched results controller.
	*/
	// Create the fetch request for the entity.
	NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
	// Edit the entity name as appropriate.
	NSEntityDescription *entity = [NSEntityDescription entityForName:@"ManagedPuzzle" inManagedObjectContext:managedObjectContext];
	[fetchRequest setEntity:entity];
	
	// Set the batch size to a suitable number.
	[fetchRequest setFetchBatchSize:20];
	
	// Edit the sort key as appropriate.
	NSSortDescriptor *sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"createDate" ascending:NO];
	NSArray *sortDescriptors = [[NSArray alloc] initWithObjects:sortDescriptor, nil];
	
	[fetchRequest setSortDescriptors:sortDescriptors];
	
	// Edit the section name key path and cache name if appropriate.
    // nil for section name key path means "no sections".
	NSFetchedResultsController *aFetchedResultsController = [[NSFetchedResultsController alloc] initWithFetchRequest:fetchRequest managedObjectContext:managedObjectContext sectionNameKeyPath:nil cacheName:@"Root"];
    aFetchedResultsController.delegate = self;
	self.fetchedResultsController = aFetchedResultsController;
	
	[aFetchedResultsController release];
	[fetchRequest release];
	[sortDescriptor release];
	[sortDescriptors release];
	
	return fetchedResultsController;
}    


// NSFetchedResultsControllerDelegate method to notify the delegate that all section and object changes have been processed. 
- (void)controllerDidChangeContent:(NSFetchedResultsController *)controller {
	// In the simplest, most efficient, case, reload the table view.
	[self.tableView reloadData];
}

/*
 Instead of using controllerDidChangeContent: to respond to all changes, you can implement all the delegate methods to update the table view in response to individual changes.  This may have performance implications if a large number of changes are made simultaneously.

// Notifies the delegate that section and object changes are about to be processed and notifications will be sent. 
- (void)controllerWillChangeContent:(NSFetchedResultsController *)controller {
	[self.tableView beginUpdates];
}

- (void)controller:(NSFetchedResultsController *)controller didChangeSection:(id <NSFetchedResultsSectionInfo>)sectionInfo atIndex:(NSUInteger)sectionIndex forChangeType:(NSFetchedResultsChangeType)type {
	// Update the table view appropriately.
}

- (void)controller:(NSFetchedResultsController *)controller didChangeObject:(id)anObject atIndexPath:(NSIndexPath *)indexPath forChangeType:(NSFetchedResultsChangeType)type newIndexPath:(NSIndexPath *)newIndexPath {
	// Update the table view appropriately.
}

- (void)controllerDidChangeContent:(NSFetchedResultsController *)controller {
	[self.tableView endUpdates];
} 
 */


#pragma mark -
#pragma mark Memory management

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
	// Relinquish ownership of any cached data, images, etc that aren't in use.
}


- (void)dealloc
{
	[timerFormatter release];
	[fetchedResultsController release];
	[managedObjectContext release];
    [super dealloc];
}

#pragma mark Extension methods

- (NSString*)formatTimer:(NSTimeInterval)timeInterval
{
	return [self.timerFormatter stringFromDate:[NSDate dateWithTimeIntervalSinceReferenceDate:timeInterval]];
}


- (void)showHighScores
{
	HighScores* hs = [[[HighScores alloc] initWithStyle:UITableViewStylePlain] autorelease];
	hs.managedObjectContext = managedObjectContext;
	
	UINavigationController* nc = [[[UINavigationController alloc] initWithRootViewController:hs] autorelease];
	[self.navigationController presentModalViewController:nc animated:YES];
}

@end
